package com.ruoyi.business.mapper;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import java.util.List;
import com.ruoyi.business.domain.TOngoodsshelf;

/**
 * 入库任务单Mapper接口
 * 
 * @author tony.wu
 * @date 2022-05-29
 */
public interface TOngoodsshelfMapper extends BaseMapper<TOngoodsshelf>
{
    /**
     * 查询入库任务单
     * 
     * @param lotId 入库任务单主键
     * @return 入库任务单
     */
    public TOngoodsshelf selectTOngoodsshelfByLotId(String lotId);

    /**
     * 查询入库任务单列表
     * 
     * @param tOngoodsshelf 入库任务单
     * @return 入库任务单集合
     */
    public List<TOngoodsshelf> selectTOngoodsshelfList(TOngoodsshelf tOngoodsshelf);

    /**
     * 新增入库任务单
     * 
     * @param tOngoodsshelf 入库任务单
     * @return 结果
     */
    public int insertTOngoodsshelf(TOngoodsshelf tOngoodsshelf);

    /**
     * 修改入库任务单
     * 
     * @param tOngoodsshelf 入库任务单
     * @return 结果
     */
    public int updateTOngoodsshelf(TOngoodsshelf tOngoodsshelf);

    /**
     * 删除入库任务单
     * 
     * @param lotId 入库任务单主键
     * @return 结果
     */
    public int deleteTOngoodsshelfByLotId(String lotId);

    /**
     * 批量删除入库任务单
     * 
     * @param lotIds 需要删除的数据主键集合
     * @return 结果
     */
    public int deleteTOngoodsshelfByLotIds(String[] lotIds);
}
